@@ -20,10 +20,14 @@
 
 #include "application.h"
 #include "dialog_new_database.h"
+#include "dialog_progress_creating.h"
+#include "utils.h"
 #include <libgnome/gnome-help.h> //For gnome_help_display
 #include "config.h" //For VERSION.
 #include <cstdio>
 #include <memory> //For std::auto_ptr<>
+#include <libgnomevfsmm.h>
+#include <sstream> //For stringstream.
 #include <glibmm/i18n.h>
 
 
@@ -32,19 +36,17 @@ App_Glom::App_Glom(BaseObjectType* cobject, const Glib::RefPtr<Gnome::Glade::Xml
   type_base(cobject, "Glom"),
   m_pBoxTop(0),
   m_pFrame(0),
-  m_pStatus(0)
+  m_menu_tables_ui_merge_id(0),
+  m_menu_reports_ui_merge_id(0)
 {
   //Load widgets from glade file:
   refGlade->get_widget("bakery_vbox", m_pBoxTop);
   refGlade->get_widget_derived("vbox_frame", m_pFrame); //This one is derived. There's a lot happening here.
-  refGlade->get_widget("label_hint", m_pStatus);
- 
+
   add_mime_type("application/x-glom"); //TODO: make this actually work - we need to register it properly.
-  
+
   //Hide the toolbar because it doesn't contain anything useful for this app.
   m_HandleBox_Toolbar.hide();
-  
-  show_all_children();
 }
 
 App_Glom::~App_Glom()
@@ -64,7 +66,7 @@ bool App_Glom::init(const Glib::ustring& document_uri)
 
   //Hide the toolbar because it doesn't contain anything useful for this app.
   //m_HandleBox_Toolbar.hide();
-  
+
   if(document_uri.empty())
   {
     Document_Glom* pDocument = static_cast<Document_Glom*>(get_document());
@@ -87,7 +89,7 @@ bool App_Glom::init(const Glib::ustring& document_uri)
 void App_Glom::init_layout()
 {
   //We override this method so that we can put everything in the vbox from the glade file, instead of the vbox from App_Gtk.
-  
+
   //Add menu bar at the top:
   //These were defined in init_uimanager().
   Gtk::MenuBar* pMenuBar = static_cast<Gtk::MenuBar*>(m_refUIManager->get_widget("/Bakery_MainMenu"));
@@ -134,7 +136,7 @@ void App_Glom::init_menus_file()
 {
   //Overridden to remove the Save and Save-As menu items,
   //because all changes are saved immediately and automatically.
-  
+
   // File menu
 
   //Build actions:
@@ -149,6 +151,14 @@ void App_Glom::init_menus_file()
   m_refFileActionGroup->add(Gtk::Action::create("BakeryAction_File_Open", Gtk::Stock::OPEN),
                         sigc::mem_fun((App_WithDoc&)*this, &App_WithDoc::on_menu_file_open));
 
+  Glib::RefPtr<Gtk::Action> action = Gtk::Action::create("BakeryAction_File_SaveAsExample", _("Save As Example"));
+  m_listDeveloperActions.push_back(action); 
+  m_refFileActionGroup->add(action,
+                        sigc::mem_fun((App_Glom&)*this, &App_Glom::on_menu_file_save_as_example));
+
+  m_refFileActionGroup->add(Gtk::Action::create("BakeryAction_Menu_File_Export", _("_Export")),
+                        sigc::mem_fun(*m_pFrame, &Frame_Glom::on_menu_file_export));
+
   m_refFileActionGroup->add(Gtk::Action::create("GlomAction_File_Print", Gtk::Stock::PRINT),
                         sigc::mem_fun(*m_pFrame, &Frame_Glom::on_menu_file_print) );
 
@@ -169,6 +179,8 @@ void App_Glom::init_menus_file()
     "        <menuitem action='BakeryAction_File_Open' />"
     "        <menu action='BakeryAction_Menu_File_RecentFiles'>"
     "        </menu>"
+    "        <menuitem action='BakeryAction_File_SaveAsExample' />"
+    "        <menuitem action='BakeryAction_Menu_File_Export' />"
     "        <separator/>"
     "        <menuitem action='GlomAction_File_Print' />"
     "        <separator/>"
@@ -193,9 +205,9 @@ void App_Glom::init_menus()
 
   //Build actions:
   m_refActionGroup_Others = Gtk::ActionGroup::create("GlomOthersActions");
-  
-  //"Navigate" menu:
-  m_refActionGroup_Others->add( Gtk::Action::create("Glom_Menu_Navigate", _("_Navigate")) );
+
+  //"Tables" menu:
+  m_refActionGroup_Others->add( Gtk::Action::create("Glom_Menu_Tables", _("_Tables")) );
 
 //  Glib::RefPtr<Gtk::Action> action = Gtk::Action::create("GlomAction_Menu_Navigate_Database", _("_Database"));
 //  m_listDeveloperActions.push_back(action);
@@ -203,9 +215,19 @@ void App_Glom::init_menus()
 //                        sigc::mem_fun(*m_pFrame, &Frame_Glom::on_menu_Navigate_Database) );
 
 
-  Glib::RefPtr<Gtk::Action> action = Gtk::Action::create("GlomAction_Menu_Navigate_Table", _("_Table"));
+  Glib::RefPtr<Gtk::Action> action = Gtk::Action::create("GlomAction_Menu_EditTables", _("_Edit Tables"));
   m_refActionGroup_Others->add(action,
-                        sigc::mem_fun(*m_pFrame, &Frame_Glom::on_menu_Navigate_Table) );
+                        sigc::mem_fun(*m_pFrame, &Frame_Glom::on_menu_Tables_EditTables) );
+  m_listDeveloperActions.push_back(action);
+
+
+  //"Reports" menu:
+  m_refActionGroup_Others->add( Gtk::Action::create("Glom_Menu_Reports", _("_Reports")) );
+
+  action = Gtk::Action::create("GlomAction_Menu_EditReports", _("_Edit Reports"));
+  m_refActionGroup_Others->add(action,
+                        sigc::mem_fun(*m_pFrame, &Frame_Glom::on_menu_Tables_EditReports) );
+  m_listDeveloperActions.push_back(action);
 
   //"UserLevel" menu:
   m_refActionGroup_Others->add(Gtk::Action::create("Glom_Menu_userlevel", _("_User Level")));
@@ -228,18 +250,29 @@ void App_Glom::init_menus()
   m_action_mode_data = Gtk::RadioAction::create(group_mode, "GlomAction_Menu_Mode_Data", _("D_ata"));
   m_refActionGroup_Others->add(m_action_mode_data,
                         sigc::mem_fun(*m_pFrame, &Frame_Glom::on_menu_Mode_Data) );
-  m_refActionGroup_Others->add( Gtk::RadioAction::create(group_mode, "GlomAction_Menu_Mode_Find", _("_Find")),  Gtk::AccelKey("<control>F"),
+
+  m_action_mode_find = Gtk::RadioAction::create(group_mode, "GlomAction_Menu_Mode_Find", _("_Find"));
+  m_refActionGroup_Others->add(m_action_mode_find,  Gtk::AccelKey("<control>F"),
                         sigc::mem_fun(*m_pFrame, &Frame_Glom::on_menu_Mode_Find) );
 
   action = Gtk::Action::create("Glom_Menu_Developer", _("_Developer"));
   m_listDeveloperActions.push_back(action);
   m_refActionGroup_Others->add(action);
 
+  action = Gtk::Action::create("GlomAction_Menu_Developer_Database_Preferences", _("_Database Preferences"));
+  m_listDeveloperActions.push_back(action);  
+  m_refActionGroup_Others->add(action, sigc::mem_fun(*m_pFrame, &Frame_Glom::on_menu_developer_database_preferences) );
+
+
   action = Gtk::Action::create("GlomAction_Menu_Developer_Fields", _("_Fields"));
   m_listDeveloperActions.push_back(action);  
   m_refActionGroup_Others->add(action, sigc::mem_fun(*m_pFrame, &Frame_Glom::on_menu_developer_fields) );
 
-  action = Gtk::Action::create("GlomAction_Menu_Developer_Relationships", _("_Relationships"));
+  action = Gtk::Action::create("GlomAction_Menu_Developer_RelationshipsOverview", _("_Relationships Overview"));
+  m_listDeveloperActions.push_back(action);
+  m_refActionGroup_Others->add(action, sigc::mem_fun(*m_pFrame, &Frame_Glom::on_menu_developer_relationships_overview) );
+
+  action = Gtk::Action::create("GlomAction_Menu_Developer_Relationships", _("_Relationships for this Table"));
   m_listDeveloperActions.push_back(action);
   m_refActionGroup_Others->add(action, sigc::mem_fun(*m_pFrame, &Frame_Glom::on_menu_developer_relationships) );
 
@@ -247,16 +280,14 @@ void App_Glom::init_menus()
   m_listDeveloperActions.push_back(action);
   m_refActionGroup_Others->add(action, sigc::mem_fun(*m_pFrame, &Frame_Glom::on_menu_developer_users));
 
+  action = Gtk::Action::create("GlomAction_Menu_Developer_Reports", _("R_eports"));
+  m_listDeveloperActions.push_back(action);
+  m_refActionGroup_Others->add(action, sigc::mem_fun(*m_pFrame, &Frame_Glom::on_menu_developer_reports));
+
   action = Gtk::Action::create("GlomAction_Menu_Developer_Layout", _("_Layout"));
   m_listDeveloperActions.push_back(action);
   m_refActionGroup_Others->add(action, sigc::mem_fun(*m_pFrame, &Frame_Glom::on_menu_developer_layout));
 
-  /*
-  action = Gtk::Action::create("GlomAction_Menu_Developer_RecreateStructure", _("_Recreate Database Structure"));
-  m_listDeveloperActions.push_back(action);
-  m_refActionGroup_Others->add(action, sigc::mem_fun(*m_pFrame, &Frame_Glom::on_menu_developer_recreate_structure));
-  */
-  
   m_refUIManager->insert_action_group(m_refActionGroup_Others);
 
   //Build part of the menu structure, to be merged in by using the "Bakery_MenuPH_Others" placeholder:
@@ -264,8 +295,15 @@ void App_Glom::init_menus()
     "<ui>"
     "  <menubar name='Bakery_MainMenu'>"
     "    <placeholder name='Bakery_MenuPH_Others'>"
-    "      <menu action='Glom_Menu_Navigate'>"
-    "        <menuitem action='GlomAction_Menu_Navigate_Table' />"
+    "      <menu action='Glom_Menu_Tables'>"
+    "        <placeholder name='Menu_Tables_Dynamic' />"
+    "        <separator />"
+    "        <menuitem action='GlomAction_Menu_EditTables' />"
+    "     </menu>"
+    "     <menu action='Glom_Menu_Reports'>"
+    "        <placeholder name='Menu_Reports_Dynamic' />"
+    "        <separator />"
+    "        <menuitem action='GlomAction_Menu_EditReports' />"
     "     </menu>"
     "      <menu action='Glom_Menu_Mode'>"
     "        <menuitem action='GlomAction_Menu_Mode_Data' />"
@@ -276,19 +314,25 @@ void App_Glom::init_menus()
     "        <menuitem action='GlomAction_Menu_userlevel_Operator' />"
     "      </menu>"
     "      <menu action='Glom_Menu_Developer'>"
+    "        <menuitem action='GlomAction_Menu_Developer_Database_Preferences' />"
     "        <menuitem action='GlomAction_Menu_Developer_Fields' />"
-    "        <menuitem action='GlomAction_Menu_Developer_Relationships' />"    
+    "        <menuitem action='GlomAction_Menu_Developer_Relationships' />"
     "        <menuitem action='GlomAction_Menu_Developer_Layout' />"
     "        <menuitem action='GlomAction_Menu_Developer_Users' />"
+    "        <menuitem action='GlomAction_Menu_Developer_Reports' />"
     "      </menu>"
     "    </placeholder>"
     "  </menubar>"
     "</ui>";
 
+/*  "        <menuitem action='GlomAction_Menu_Developer_RelationshipsOverview' />" */
+
   //Add menu:
   add_ui_from_string(ui_description);
 
   init_menus_help();
+
+  fill_menu_tables();
 }
 
 void App_Glom::on_menu_userlevel_developer()
@@ -344,13 +388,58 @@ bool App_Glom::on_document_load()
 
     //Disable/Enable actions, depending on userlevel:
     pDocument->emit_userlevel_changed();
-    
+
     if(pDocument->get_connection_database().empty()) //If it is a new (default) document.
     {
       //offer_new_or_existing();
     }
     else
     {
+      //Prevent saving until we are sure that everything worked.
+      //This also stops us from losing the example data as soon as we say the new file (created from the example) is not an example.
+      pDocument->set_allow_autosave(false);
+
+      const bool is_example = pDocument->get_is_example_file();
+      if(pDocument->get_is_example_file())
+      {
+          Gtk::MessageDialog dialog(Bakery::App_Gtk::util_bold_message(_("Creating from example file.")), true,  Gtk::MESSAGE_INFO, Gtk::BUTTONS_NONE);
+          dialog.set_secondary_text(_("To use this example file you must save an editable copy of the file. A new database will also be created on the server.")); //TODO: Better explanation.
+          dialog.set_transient_for(*this);
+
+          dialog.add_button(Gtk::Stock::CANCEL, Gtk::RESPONSE_CANCEL);
+          dialog.add_button(Gtk::Stock::SAVE_AS, Gtk::RESPONSE_OK);
+
+          const int response = dialog.run();
+          dialog.hide();
+          if(response == Gtk::RESPONSE_CANCEL)
+            return false;
+          else
+          {
+            offer_saveas();
+            if(get_operation_cancelled())
+              return false;
+            else
+              pDocument->set_is_example_file(false);
+          }
+      }
+
+      //Warn about read-only files, because users will otherwise wonder why they can't use Developer mode:
+      Document_Glom::userLevelReason reason = Document_Glom::USER_LEVEL_REASON_UNKNOWN;
+      const AppState::userlevels userlevel = pDocument->get_userlevel(reason);
+      if( (userlevel == AppState::USERLEVEL_OPERATOR) && (reason == Document_Glom::USER_LEVEL_REASON_FILE_READ_ONLY) )
+      {
+        Gtk::MessageDialog dialog(Bakery::App_Gtk::util_bold_message(_("Opening Read-Only File.")), true,  Gtk::MESSAGE_INFO, Gtk::BUTTONS_NONE);
+        dialog.set_secondary_text(_("This file is read only, so you will not be able to enter Developer mode to make design changes."));
+        dialog.set_transient_for(*this);
+        dialog.add_button(Gtk::Stock::CANCEL, Gtk::RESPONSE_CANCEL);
+        dialog.add_button(_("Continue without Developer Mode"), Gtk::RESPONSE_OK); //arbitrary response code.
+
+        const int response = dialog.run();
+        dialog.hide();
+        if(response == Gtk::RESPONSE_CANCEL)
+          return false;
+      }
+
       //Read the connection information from the document:
       ConnectionPool* connection_pool = ConnectionPool::get_instance();
       if(!connection_pool)
@@ -368,94 +457,82 @@ bool App_Glom::on_document_load()
         //Attempt to connect to the specified database:
         try
         {
-          bool test = m_pFrame->connection_request_password_and_attempt();
-          if(!test)
+          bool test = false;
+          if(is_example)
+            test = m_pFrame->connection_request_password_and_choose_new_database_name();
+          else
+            test = m_pFrame->connection_request_password_and_attempt();
+
+          if(!test) //It usually throws an exception instead of returning false.
             return false; //Failed. Close the document.
-        }
-        catch(const ExceptionConnection& ex)
-        { 
-          if(ex.get_failure_type() == ExceptionConnection::FAILURE_NO_DATABASE) //This is the only FALURE_* type that connection_request_password_and_attempt() throws.
+
+          if(is_example)
           {
-            //The connection to the server is OK, but the database is not there yet.
-            //Ask the user if he wants to create it.
-            //For instance, it might be an example document, whose database does not exist on his server yet.
-
-            Glib::RefPtr<Gnome::Glade::Xml> refXml = Gnome::Glade::Xml::create(GLOM_GLADEDIR "glom.glade", "dialog_recreate_database");
-            Gtk::Dialog* dialog = 0;
-            refXml->get_widget("dialog_recreate_database", dialog);
-            if(dialog)
+            //Create the example database:
+            //connection_request_password_and_choose_new_database_name() has already change the database name to a new unused one:
+            bool user_cancelled = false;
+            const bool test = recreate_database(user_cancelled);
+            if(!test)
             {
-              int response = dialog->run();
-              dialog->set_transient_for(*this);
-              delete dialog;
-              dialog = 0;
-
-              if(response == Gtk::RESPONSE_CANCEL)
-                return false; //Close the document.
-              else
+              //If the database was not successfully recreated:
+              if(!user_cancelled)
               {
-                bool user_cancelled = false;
-                bool test = recreate_database(user_cancelled);
-
-                if(test)
+                //Tell the user:
+                Gtk::Dialog* dialog = 0;
+                try
                 {
-                  //If the database was successfully recreated.
-
-                  //Warn about read-only files, such as installed example files.
-                  Document_Glom::userLevelReason reason = Document_Glom::USER_LEVEL_REASON_UNKNOWN;
-                  AppState::userlevels userlevel = pDocument->get_userlevel(reason);
-                  if( (userlevel == AppState::USERLEVEL_OPERATOR) && (reason == Document_Glom::USER_LEVEL_REASON_FILE_READ_ONLY) )
-                  {
-                    Gtk::MessageDialog dialog(Bakery::App_Gtk::util_bold_message(_("Creating from read-only file.")), true,  Gtk::MESSAGE_WARNING);
-                    dialog.set_secondary_text(_("This file is read only, so you will not be able to enter Developer mode to make design changes. Maybe this file is an installed example file. Therefore, you might want to create your own writeable copy of this file."));
-                    dialog.set_transient_for(*this);
-                    dialog.run();
-                    //TODO: Store a magic number in the database (a special table) and the file to check for mismatches.
-                  }
+                  Glib::RefPtr<Gnome::Glade::Xml> refXml = Gnome::Glade::Xml::create(GLOM_GLADEDIR "glom.glade", "dialog_error_create_database");
+                  refXml->get_widget("dialog_error_create_database", dialog);
+                  dialog->set_transient_for(*this);
+                  dialog->run();
+                  delete dialog;
                 }
-                else
+                catch(const Gnome::Glade::XmlError& ex)
                 {
-                  //If the database was not successfully recreated:
-
-                  if(!user_cancelled)
-                  {
-                    //Tell the user:
-                    Gtk::Dialog* dialog = 0;
-                    try
-                    {
-                      Glib::RefPtr<Gnome::Glade::Xml> refXml = Gnome::Glade::Xml::create(GLOM_GLADEDIR "glom.glade", "dialog_error_create_database");
-  
-                      refXml->get_widget("dialog_error_create_database", dialog);
-                    }
-                    catch(const Gnome::Glade::XmlError& ex)
-                    {
-                      std::cerr << ex.what() << std::endl;
-                    }
-                  }
-
-                  return false;
+                  std::cerr << ex.what() << std::endl;
                 }
-
-                //TODO: Recreate the database, using the information saved in the document:
-                //Frame_Glom::show_ok_dialog(_("Database recreation"), _("Sorry. This functionality has not been implemented yet."), *this);
-                //return false; //Give up on this document.
               }
+
+              return false;
             }
           }
         }
+        catch(const ExceptionConnection& ex)
+        {
+          if(ex.get_failure_type() == ExceptionConnection::FAILURE_NO_DATABASE) //This is the only FAILURE_* type that connection_request_password_and_attempt() throws.
+          {
+            if(!is_example)
+            {
+              //The connection to the server is OK, but the database is not there yet.
+              Frame_Glom::show_ok_dialog(_("Database Not Found On Server"), _("The database could not be found on the server. Please consult your system administrator."), *this, Gtk::MESSAGE_ERROR);
+            }
+            else
+              std::cerr << "App_Glom::on_document_load(): unexpected ExceptionConnection when opening example." << std::endl;
+          }
+          else
+            std::cerr << "App_Glom::on_document_load(): unexpected ExceptionConnection failure type." << std::endl;
+        }
 
         //Switch to operator mode when opening new documents:
         pDocument->set_userlevel(AppState::USERLEVEL_OPERATOR);
 
+        m_pFrame->show_system_name();
+
         //Open default table, or show list of tables instead:
-        m_pFrame->do_menu_Navigate_Table(true /* open the default if there is one */);    
+        m_pFrame->do_menu_Navigate_Table(true /* open the default if there is one */);
+
       }
     }
 
+    //List the non-hidden tables in the menu:
+    fill_menu_tables();
+
+    pDocument->set_allow_autosave(true);
     return true; //Loading of the document into the application succeeded.
   }
 }
 
+/*
 void App_Glom::statusbar_set_text(const Glib::ustring& strText)
 {
   m_pStatus->set_text(strText);
@@ -465,6 +542,7 @@ void App_Glom::statusbar_clear()
 {
   statusbar_set_text("");
 }
+*/
 
 
 void App_Glom::on_userlevel_changed(AppState::userlevels /* userlevel */)
@@ -518,6 +596,30 @@ bool App_Glom::offer_new_or_existing()
   {
     on_menu_file_open();
 
+    //Check that a document was opened:
+    Document_Glom* document = dynamic_cast<Document_Glom*>(get_document());
+    if(document->get_file_uri().empty())
+    {
+      //Ask again:
+      return offer_new_or_existing();
+    }
+  }
+  if(response_id == 3) //Open Example
+  {
+    //Based on on_menu_file_open();
+
+    //Display File Open dialog and respond to choice:
+
+    //Bring document window to front, to make it clear which document is being changed:
+    ui_bring_to_front();
+
+    //Ask user to choose file to open:
+    //g_warning("GLOM_EXAMPLES_DIR=%s", GLOM_EXAMPLES_DIR);
+    Glib::ustring file_uri = ui_file_select_open(GLOM_EXAMPLES_DIR);
+    if(!file_uri.empty())
+      open_document(file_uri);
+
+
     //Check that a document was opened:
     Document_Glom* document = dynamic_cast<Document_Glom*>(get_document());
     if(document->get_file_uri().empty())
@@ -552,7 +654,7 @@ bool App_Glom::offer_new_or_existing()
 
           //Set suitable defaults:
           const Glib::ustring filename = document->get_name(); //Get the filename without the path and extension.
-          dialog->set_input(filename, Base_DB::util_title_from_string( filename ) );
+          dialog->set_input( Base_DB::util_title_from_string( filename ) ); //Start with something suitable.
 
           bool keep_asking = true;
           while(keep_asking)
@@ -561,9 +663,12 @@ bool App_Glom::offer_new_or_existing()
 
             if(response == Gtk::RESPONSE_OK)
             {
-              Glib::ustring db_name;
               Glib::ustring db_title;
-              dialog->get_input(db_name, db_title);
+              dialog->get_input(db_title);
+
+              //Create a database name based on the title.
+              //The user will (almost) never see this anyway but it's nicer than using a random number:
+              Glib::ustring db_name = GlomUtils::create_name_from_title(db_title);
 
               //Prefix glom_ to the database name, so it's more obvious
               //for the system administrator.
@@ -572,21 +677,31 @@ bool App_Glom::offer_new_or_existing()
 
               if(!db_name.empty()) //The dialog prevents this anyway.
               {
-                bool db_created = m_pFrame->create_database(db_name);
+                //Connect to the server and choose a variation of this db_name that does not exist yet:
+                document->set_connection_database(db_name);
+                const bool connected = m_pFrame->connection_request_password_and_choose_new_database_name();
+                if(!connected)
+                  return false;
+
+                const bool db_created = m_pFrame->create_database(document->get_connection_database(), db_title, false /* do not request password */);
                 if(db_created)
                 {
                   keep_asking = false;
-                
-                  document->set_connection_database(db_name); //Select the database that was just created.
 
+                  //document->set_connection_database(db_name); //Select the database that was just created.
+
+                  /*
                   ConnectionPool* connection_pool = ConnectionPool::get_instance();
                   if(connection_pool)
                   {
                     connection_pool->set_database(db_name); //The rest has been set while creating the database.
                   }
+                  */
 
+                  const Glib::ustring database_name_used = document->get_connection_database();
+                  ConnectionPool::get_instance()->set_database(database_name_used);
                   document->set_database_title(db_title);
-                  m_pFrame->set_databases_selected(db_name);
+                  m_pFrame->set_databases_selected(database_name_used);
                 }
                 else
                 {
@@ -602,13 +717,13 @@ bool App_Glom::offer_new_or_existing()
             else
             {
               return false; //The user cancelled.
-            }            
+            }
           } /* while() */
-            
+
           return true; //File successfully created.
         }
        }
-  
+
       return false; //Creation of new document failed.
     }
     else
@@ -634,6 +749,11 @@ void App_Glom::set_mode_data()
   m_action_mode_data->activate();
 }
 
+void App_Glom::set_mode_find()
+{
+  m_action_mode_data->activate();
+}
+
 
 void App_Glom::init_menus_help()
 {
@@ -692,31 +812,57 @@ bool App_Glom::recreate_database(bool& user_cancelled)
   catch(const ExceptionConnection& ex)
   {
     if(ex.get_failure_type() == ExceptionConnection::FAILURE_NO_SERVER)
-    { 
+    {
       user_cancelled = true; //Eventually, the user will cancel after retrying.
       g_warning("App_Glom::recreate_database(): Failed because connection to server failed, without specifying a databse.");
       return false;
     }
 
-    //Otherwise continue, because we expected connect() to fail if the db does not exist yet.
+    //Otherwise continue, because we _expected_ connect() to fail if the db does not exist yet.
   }
 
- //Create the database:
+  //Show the user that something is happening, because the INSERTS might take time.
+  //TOOD: This doesn't actually show up until near the end, even with Gtk::Main::instance()->iteration().
+  std::auto_ptr<Dialog_ProgressCreating> dialog_progress;
+  Glib::RefPtr<Gnome::Glade::Xml> refXml = Gnome::Glade::Xml::create(GLOM_GLADEDIR "glom.glade", "window_progress_creating_from_example");
+  if(refXml)
+  {
+    Dialog_ProgressCreating* dialog_progress_temp = 0;
+    refXml->get_widget_derived("window_progress_creating_from_example", dialog_progress_temp);
+    if(dialog_progress_temp)
+    {
+      dialog_progress.reset(dialog_progress_temp); //The dialog will be deleted (and hidden) when this function returns.
+
+      dialog_progress->set_transient_for(*dialog_progress_temp);
+      dialog_progress->show();
+
+      //Ensure that the dialog is shown, instead of waiting for the application to be idle:
+      while(Gtk::Main::instance()->events_pending())
+        Gtk::Main::instance()->iteration();
+    }
+  }
+
+  dialog_progress->pulse();
+
+  //Create the database: (This will show a connection dialog)
   connection_pool->set_database( Glib::ustring() );
-  bool db_created = m_pFrame->create_database(db_name, false /* Don't ask for password etc again. */);
+  bool db_created = m_pFrame->create_database(db_name, pDocument->get_database_title(), false /* Don't ask for password etc again. */);
 
   if(!db_created)
   {
     return false;
   }
   else
-    connection_pool->set_database(db_name); //Specify the new database when connection from now on.
+    connection_pool->set_database(db_name); //Specify the new database when connecting from now on.
+
+  dialog_progress->pulse();
+  Bakery::BusyCursor(*this);
 
   sharedptr<SharedConnection> sharedconnection;
   try
   {
     sharedconnection = connection_pool->connect();
-    connection_pool->set_database(db_name); //The database was successfully created, so specify it when connection from now on.
+    connection_pool->set_database(db_name); //The database was successfully created, so specify it when connecting from now on.
   }
   catch(const ExceptionConnection& ex)
   {
@@ -724,80 +870,54 @@ bool App_Glom::recreate_database(bool& user_cancelled)
     return false;
   }
 
-  Bakery::BusyCursor(*this);
+  dialog_progress->pulse();
 
   //Create each table:
   Document_Glom::type_listTableInfo tables = pDocument->get_tables();
   for(Document_Glom::type_listTableInfo::const_iterator iter = tables.begin(); iter != tables.end(); ++iter)
   {
     const TableInfo& table_info = *iter;
-    const Glib::ustring table_name = table_info.get_name();
 
     //Create SQL to describe all fields in this table:
     Glib::ustring sql_fields;
-    Document_Glom::type_vecFields fields = pDocument->get_table_fields(table_name);
-    for(Document_Glom::type_vecFields::const_iterator iter = fields.begin(); iter != fields.end(); ++iter)
-    {
-      //Create SQL to describe this field:
-      Field field = *iter;
-
-      //The field has no gda type, so we set that:
-      //This usually comes from the database, but that's a bit strange.
-      Gnome::Gda::FieldAttributes info = field.get_field_info();
-      info.set_gdatype( Field::get_gda_type_for_glom_type(field.get_glom_type()) );
-      field.set_field_info(info);
-
-      Glib::ustring sql_field_description = field.get_name() + " " + field.get_sql_type();
-
-      if(field.get_primary_key())
-        sql_field_description += " NOT NULL  PRIMARY KEY";
-
-      //Append it:
-      if(!sql_fields.empty())
-        sql_fields += ", ";
-
-      sql_fields += sql_field_description;
-    }
-
-    if(sql_fields.empty())
-    {
-      g_warning("App_Glom::recreate_database(): sql_fields is empty.");
-    }
-
-    //Actually create the table
-    bool table_creation_succeeded = true;
-    try
-    {
-      Glib::RefPtr<Gnome::Gda::DataModel> data_model = m_pFrame->Query_execute( "CREATE TABLE \"" + table_name + "\" (" + sql_fields + ")" );
-      if(!data_model)
-        table_creation_succeeded = false;
-    }
-    catch(const ExceptionConnection& ex)
-    {
-      table_creation_succeeded = false;
-    }
+    Document_Glom::type_vecFields fields = pDocument->get_table_fields(table_info.get_name());
 
+    dialog_progress->pulse();
+    const bool table_creation_succeeded = m_pFrame->create_table(table_info, fields);
+    dialog_progress->pulse();
     if(!table_creation_succeeded)
     {
       g_warning("App_Glom::recreate_database(): CREATE TABLE failed with the newly-created database.");
       return false;
     }
+    else
+    {
+      dialog_progress->pulse();
+      m_pFrame->add_standard_tables(); //Add internal, hidden, tables.
+
+      //Create the developer group, and make this user a member of it:
+      //If we got this far then the user must really have developer privileges already:
+      dialog_progress->pulse();
+      m_pFrame->add_standard_groups();
+
+      Glib::ustring strQuery = "ALTER GROUP " GLOM_STANDARD_GROUP_NAME_DEVELOPER " ADD USER " + connection_pool->get_user();
+      m_pFrame->Query_execute(strQuery);
+
+      //Add any example data to the table:
+      dialog_progress->pulse();
+      const bool table_insert_succeeded = m_pFrame->insert_example_data(table_info.get_name());
+      if(!table_insert_succeeded)
+      {
+        g_warning("App_Glom::recreate_database(): INSERT of example data failed with the newly-created database.");
+        return false;
+      }
+    }
 
   } //for(tables)
 
-
-  //Create the developer group, and make this user a member of it:
-  //If we got this far then the user must really have developer privileges already:
-  m_pFrame->add_standard_groups();
-
-  Glib::ustring strQuery = "ALTER GROUP " GLOM_STANDARD_GROUP_NAME_DEVELOPER " ADD USER " + connection_pool->get_user();
-  m_pFrame->Query_execute(strQuery);
-
   return true; //All tables created successfully.
 }
 
-  
-
 AppState::userlevels App_Glom::get_userlevel() const
 {
   const Document_Glom* document = dynamic_cast<const Document_Glom*>(get_document());
@@ -813,7 +933,7 @@ void App_Glom::add_developer_action(const Glib::RefPtr<Gtk::Action>& refAction)
 {
   //Prevent it from being added twice:
   remove_developer_action(refAction);
-  
+
   m_listDeveloperActions.push_back(refAction);
 }
 
@@ -828,3 +948,234 @@ void App_Glom::remove_developer_action(const Glib::RefPtr<Gtk::Action>& refActio
     }
   }
 }
+
+void App_Glom::fill_menu_tables()
+{
+  //TODO: There must be a better way than building a ui_string like this:
+
+  m_listNavTableActions.clear();
+  if(m_menu_tables_ui_merge_id)
+    m_refUIManager->remove_ui(m_menu_tables_ui_merge_id);
+
+  m_refNavTablesActionGroup = Gtk::ActionGroup::create("NavTablesActions");
+
+  Glib::ustring ui_description =
+    "<ui>"
+    "  <menubar name='Bakery_MainMenu'>"
+    "    <placeholder name='Bakery_MenuPH_Others'>"
+    "      <menu action='Glom_Menu_Tables'>"
+    "        <placeholder name='Menu_Tables_Dynamic'>";
+
+  Document_Glom* document = dynamic_cast<Document_Glom*>(get_document());
+  const Document_Glom::type_listTableInfo tables = document->get_tables();
+  for(Document_Glom::type_listTableInfo::const_iterator iter = tables.begin(); iter != tables.end(); ++iter)
+  {
+    const TableInfo& table_info = *iter;
+    if(!table_info.m_hidden)
+    {
+      const Glib::ustring action_name = "NavTableAction_" + table_info.get_name();
+
+      ui_description += "<menuitem action='" + action_name + "' />";
+
+      Glib::RefPtr<Gtk::Action> refAction = Gtk::Action::create(action_name, table_info.get_title_or_name());
+      m_refNavTablesActionGroup->add(refAction,
+        sigc::bind( sigc::mem_fun(*m_pFrame, &Frame_Glom::on_box_tables_selected), table_info.m_name) );
+
+      m_listNavTableActions.push_back(refAction);
+
+      //m_refUIManager->add_ui(merge_id, path, table_info.m_title, refAction, UI_MANAGER_MENUITEM);
+    }
+  }
+
+  m_refUIManager->insert_action_group(m_refNavTablesActionGroup);
+
+
+  ui_description +=
+    "     </placeholder>"
+    "    </menu>"
+    "    </placeholder>"
+    "  </menubar>"
+    "</ui>";
+
+  //Add menus:
+  try
+  {
+    m_menu_tables_ui_merge_id = m_refUIManager->add_ui_from_string(ui_description);
+  }
+  catch(const Glib::Error& ex)
+  {
+    std::cerr << " App_Glom::fill_menu_tables(): building menus failed: " <<  ex.what();
+  }
+}
+
+
+void App_Glom::fill_menu_reports(const Glib::ustring& table_name)
+{
+  //TODO: There must be a better way than building a ui_string like this:
+
+  m_listNavReportActions.clear();
+  if(m_menu_reports_ui_merge_id)
+    m_refUIManager->remove_ui(m_menu_reports_ui_merge_id);
+
+  m_refNavReportsActionGroup = Gtk::ActionGroup::create("NavReportsActions");
+
+  Glib::ustring ui_description =
+    "<ui>"
+    "  <menubar name='Bakery_MainMenu'>"
+    "    <placeholder name='Bakery_MenuPH_Others'>"
+    "     <menu action='Glom_Menu_Reports'>"
+    "        <placeholder name='Menu_Reports_Dynamic'>";
+
+  Document_Glom* document = dynamic_cast<Document_Glom*>(get_document());
+  const Document_Glom::type_listReports tables = document->get_report_names(table_name);
+  for(Document_Glom::type_listReports::const_iterator iter = tables.begin(); iter != tables.end(); ++iter)
+  {
+    Report report;
+    bool found =  document->get_report(table_name, *iter, report);
+    if(found)
+    {
+      const Glib::ustring report_name = report.get_name();
+      if(!report_name.empty())
+      {
+        const Glib::ustring action_name = "NavReportAction_" + report_name;
+
+        ui_description += "<menuitem action='" + action_name + "' />";
+
+        Glib::RefPtr<Gtk::Action> refAction = Gtk::Action::create(action_name, report.get_title_or_name());
+        m_refNavReportsActionGroup->add(refAction,
+          sigc::bind( sigc::mem_fun(*m_pFrame, &Frame_Glom::on_menu_report_selected), report.m_name) );
+
+        m_listNavReportActions.push_back(refAction);
+
+        //m_refUIManager->add_ui(merge_id, path, table_info.m_title, refAction, UI_MANAGER_MENUITEM);
+      }
+    }
+  }
+
+  m_refUIManager->insert_action_group(m_refNavReportsActionGroup);
+
+
+  ui_description +=
+    "     </placeholder>"
+    "    </menu>"
+    "    </placeholder>"
+    "  </menubar>"
+    "</ui>";
+
+  //Add menus:
+  try
+  {
+    m_menu_reports_ui_merge_id = m_refUIManager->add_ui_from_string(ui_description);
+  }
+  catch(const Glib::Error& ex)
+  {
+    std::cerr << " App_Glom::fill_menu_reports(): building menus failed: " <<  ex.what();
+  }
+}
+
+void App_Glom::on_menu_file_save_as_example()
+{
+  //Based on the implementation of Bakery::App_WithDoc::on_menu_file_saveas()
+
+  //Display File Save dialog and respond to choice:
+
+  //Bring document window to front, to make it clear which document is being saved:
+  //This doesn't work: TODO.
+  ui_bring_to_front();
+
+  //Show the save dialog:
+  Document_Glom* document = dynamic_cast<Document_Glom*>(get_document());
+  const Glib::ustring& file_uriOld = document->get_file_uri();
+  Glib::ustring file_uri = ui_file_select_save(file_uriOld); //Also asks for overwrite confirmation.
+  if(!file_uri.empty())
+  {
+    //Enforce the file extension:
+    file_uri = document->get_file_uri_with_extension(file_uri);
+
+    bool bUseThisFileUri = true;
+
+    //Check whether file exists already:
+    {
+      // Try to open the input file.
+      Gnome::Vfs::Handle read_handle;
+      try
+      {
+        read_handle.open(file_uri, Gnome::Vfs::OPEN_READ);
+
+        //It does (there was no exception), so ask the user to confirm overwrite:
+        const bool bOverwrite = true; //The FileChooser asked already. ui_ask_overwrite(file_uri);
+
+        //Respond to button that was clicked:
+        bUseThisFileUri = bOverwrite;
+      }
+      catch(const Gnome::Vfs::exception& ex)
+      {
+        bUseThisFileUri = true; //It does not exist.
+      }
+    }
+
+    //Save to this filepath:
+    if(bUseThisFileUri)
+    {
+      //Prevent saving while we modify the document:
+      document->set_allow_autosave(false);
+
+      document->set_file_uri(file_uri, true); //true = enforce file extension
+      document->set_is_example_file();
+
+      //Save all data from all tables into the document:
+      Document_Glom::type_listTableInfo list_table_info = document->get_tables();
+      for(Document_Glom::type_listTableInfo::const_iterator iter = list_table_info.begin(); iter != list_table_info.end(); ++iter)
+      {
+        const Glib::ustring table_name = iter->get_name();
+
+        //const type_vecFields vec_fields = document->get_table_fields(table_name);
+
+        //export_data_to_stream() needs a type_mapLayoutGroupSequence;
+        Document_Glom::type_mapLayoutGroupSequence sequence = document->get_data_layout_groups_default("list", table_name);
+
+        //std::cout << "debug: table_name=" << table_name << std::endl;
+
+        Glib::ustring row_text;
+        m_pFrame->export_data_to_string(row_text, table_name, sequence);
+        //std::cout << "  debug after row_text=" << row_text << std::endl;
+
+        document->set_table_example_data(table_name, row_text);
+      }
+
+      document->set_allow_autosave(true);
+
+      bool bTest  = document->save();
+
+      if(!bTest)
+      {
+        ui_warning(_("Save failed."), _("There was an error while saving the file. Your changes have not been saved."));
+      }
+      else
+      {
+        //Disable Save and SaveAs menu items:
+        after_successful_save();
+      }
+
+      update_window_title();
+
+
+      //Close if this save was a result of a File|Close or File|Exit:.
+      //if(bTest && m_bCloseAfterSave) //Don't close if the save failed.
+      //{
+      //  on_menu_file_close(); //This could be the second time, but now there are no unsaved changes.
+      //}
+    }
+    else
+    {
+      //Let the user choose a different file path,
+      //because he decided not to overwrite the 1st one.
+      on_menu_file_save_as_example(); //recursive.
+    }
+  }
+  else
+  {
+    cancel_close_or_exit();
+  }
+}
+
