@@ -34,17 +34,14 @@ public:
   Box_DB_Table(BaseObjectType* cobject, const Glib::RefPtr<Gnome::Glade::Xml>& refGlade);
   virtual ~Box_DB_Table();
 
-  virtual void init_db_details(const Glib::ustring& strTableName);
-  virtual void refresh_data_from_database();
+  virtual bool init_db_details(const Glib::ustring& strTableName);
+  virtual bool refresh_data_from_database();
 
   virtual Glib::ustring get_table_name();
 
   //TODO: Put this somewhere more sensible:
   typedef std::map<Gnome::Gda::ValueType, Glib::ustring> type_map_valuetypes;
 
-
-  bool get_fields_for_table_one_field(const Glib::ustring& table_name, const Glib::ustring& field_name, Field& field) const;      
-
 protected:
 
   //virtual Glib::RefPtr<Gnome::Gda::DataModel> record_new(Gnome::Gda::Value primary_key_value);
@@ -52,10 +49,9 @@ protected:
   virtual Gnome::Gda::Value get_entered_field_data(const LayoutItem_Field& field) const;
   virtual void set_entered_field_data(const LayoutItem_Field& field, const Gnome::Gda::Value& value);
 
-  bool get_field_primary_key_for_table(const Glib::ustring table_name, Field& field) const;
   //static bool get_field_primary_key(const type_vecFields& fields, Field& field);
 
-  unsigned long get_last_auto_increment_value(const Glib::RefPtr<Gnome::Gda::DataModel>& data_model, const Glib::ustring field_name);
+  unsigned long get_last_auto_increment_value(const Glib::RefPtr<Gnome::Gda::DataModel>& data_model, const Glib::ustring& field_name);
 
 
   static type_vecFields get_fields_for_datamodel(const Glib::RefPtr<Gnome::Gda::DataModel>& data_model); 
