@@ -21,6 +21,7 @@
 #include "field.h"
 #include "../connectionpool.h"
 #include "glomconversions.h"
+#include "../utils.h"
 #include <glibmm/i18n.h>
 
 //Initialize static data:
@@ -28,6 +29,7 @@ Field::type_map_gda_type_to_glom_type Field::m_map_gda_type_to_glom_type;
 Field::type_map_glom_type_to_gda_type Field::m_map_glom_type_to_gda_type;
 Field::type_map_type_names Field::m_map_type_names;
 Field::type_map_type_names Field::m_map_type_names_ui;
+Field::type_map_conversions Field::m_map_conversions;
 bool Field::m_maps_inited = false;
 
 
@@ -39,6 +41,7 @@ Field::Field()
 
 Field::Field(const Field& src)
 {
+  //TODO_Performance: Implement this properly, without the extra copy.
   operator=(src);
 }
 
@@ -60,6 +63,8 @@ Field& Field::operator=(const Field& src)
 
   m_visible = src.m_visible;
 
+  m_default_formatting = src.m_default_formatting;
+
   return *this;
 }
 
@@ -76,7 +81,9 @@ bool Field::operator==(const Field& src) const
   bResult = bResult && (m_calculation == src.m_calculation);
 
   bResult = bResult && (m_visible == src.m_visible);
-  
+
+  bResult = bResult && (m_default_formatting == src.m_default_formatting);
+
   return bResult;
 }
 
@@ -165,10 +172,87 @@ Glib::ustring Field::get_title_or_name() const
   }
 }
 
+namespace { //anonymous
+
+//A copy of PQescapeString from the Postgres source, to avoid linking with libpql directly, 
+//and to use until we can use the latest libgda, which has an equivalent.
+
+#define SQL_STR_DOUBLE(ch)	((ch) == '\'' || (ch) == '\\')
+
+/*
+ * Escaping arbitrary strings to get valid SQL literal strings.
+ *
+ * Replaces "\\" with "\\\\" and "'" with "''".
+ *
+ * length is the length of the source string.  (Note: if a terminating NUL
+ * is encountered sooner, PQescapeString stops short of "length"; the behavior
+ * is thus rather like strncpy.)
+ *
+ * For safety the buffer at "to" must be at least 2*length + 1 bytes long.
+ * A terminating NUL character is added to the output string, whether the
+ * input is NUL-terminated or not.
+ *
+ * Returns the actual length of the output (not counting the terminating NUL).
+ */
+size_t
+Glom_PQescapeString(char *to, const char *from, size_t length)
+{
+	const char *source = from;
+	char	   *target = to;
+	size_t		remaining = length;
+
+	while (remaining > 0 && *source != '\0')
+	{
+		if (SQL_STR_DOUBLE(*source))
+			*target++ = *source;
+		*target++ = *source++;
+		remaining--;
+	}
+
+	/* Write the terminating NUL character. */
+	*target = '\0';
+
+	return target - to;
+}
+
+} //anonymous
+
+static std::string glom_escape_text(const std::string src)
+{
+  if(src.empty())
+    return "''"; //We want to ignore the concept of NULL strings, and deal only with empty strings.
+  else
+  {
+    const size_t len = src.size();
+    char* to = (char*)malloc(sizeof(char) * 2 * (len + 1)); //recommended size for to.
+    const size_t len_escaped = Glom_PQescapeString(to, src.c_str(), len);
+    if(!len_escaped)
+    {
+      std::cerr << "glom_escape_text(): Glom_PQescapeString() failed with text: " << src << std::endl;
+
+      if(to)
+        free(to);
+
+      return "''";
+    }
+    else
+    {
+      std::string escaped(to, len_escaped);
+      free(to);
+
+      //Also escape any ";" characters, because these also cause problems, at least with libgda:
+      //See bug #326325.
+      escaped = GlomUtils::string_replace(escaped, ";", "\\073");
+
+      return ("'" + escaped + "'"); //Add single-quotes. Actually escape it 
+      //std::cout << "glom_escape_text: escaped and quoted: " << str << std::endl;
+    }
+  }
+}
+
 Glib::ustring Field::sql(const Gnome::Gda::Value& value) const
 {
-//g_warning("Field::sql: glom_type=%d", get_glom_type());
-
+  //g_warning("Field::sql: glom_type=%d", get_glom_type());
 
   if(value.is_null())
   {
@@ -182,6 +266,7 @@ Glib::ustring Field::sql(const Gnome::Gda::Value& value) const
       case(TYPE_DATE):
       case(TYPE_TIME):
       case(TYPE_NUMERIC):
+      case(TYPE_IMAGE):
       {
         return "NULL";
         break;
@@ -209,7 +294,11 @@ Glib::ustring Field::sql(const Gnome::Gda::Value& value) const
       if(value.is_null())
         return "''"; //We want to ignore the concept of NULL strings, and deal only with empty strings.
       else
-        str = "'" + value.to_string() + "'"; //Add single-quotes. Actually escape it.
+      {
+        str = glom_escape_text(value.get_string());
+      }
+
+      break;
     }
     case(TYPE_DATE):
     case(TYPE_TIME):
@@ -235,6 +324,27 @@ Glib::ustring Field::sql(const Gnome::Gda::Value& value) const
 
       break;
     }
+    case(TYPE_IMAGE):
+    {
+      if(value.get_value_type() == Gnome::Gda::VALUE_TYPE_BINARY)
+      {
+        long buffer_size = 0;
+        const gpointer buffer = value.get_binary(buffer_size);
+        if(buffer && buffer_size)
+        {
+          const std::string escaped_binary_data = std::string((char*)buffer, buffer_size);
+          //Now escape that text (to convert \ to \\, for instance):
+          str = glom_escape_text(escaped_binary_data) /* has quotes */ + "::bytea";
+
+          //Use this when libgda unescapes the binary data internally, as it should: 
+          //str = "'" + GlomConversions::get_escaped_binary_data((guint8*)buffer, buffer_size) + "'::bytea";
+        }
+      }
+      else
+        g_warning("Field::sql(): glom_type is TYPE_IMAGE but gda type is not VALUE_TYPE_BINARY");
+
+      break;
+    }
     default:
     {
       str = value.to_string();
@@ -248,6 +358,55 @@ Glib::ustring Field::sql(const Gnome::Gda::Value& value) const
   return str;
 }
 
+Glib::ustring Field::sql_find(const Gnome::Gda::Value& value) const
+{
+  switch(get_glom_type())
+  {
+    case(TYPE_TEXT):
+    {
+      //% means 0 or more characters.
+      
+      if(value.is_null())
+        return "''"; //We want to ignore the concept of NULL strings, and deal only with empty strings.
+      else
+        return ("'%" + value.to_string() + "%'"); //Add single-quotes. Actually escape it.
+        
+      break;
+    }
+    case(TYPE_DATE):
+    case(TYPE_TIME):
+    case(TYPE_NUMERIC):
+    case(TYPE_BOOLEAN):
+    default:
+    {
+      return sql(value);
+      break;
+    }
+  }
+}
+
+Glib::ustring Field::sql_find_operator() const
+{
+  switch(get_glom_type())
+  {
+    case(TYPE_TEXT):
+    {
+      return "ILIKE"; //"LIKE"; ILIKE is a postgres extension for locale-dependent case-insensitive matches.
+      break;
+    }
+    case(TYPE_DATE):
+    case(TYPE_TIME):
+    case(TYPE_NUMERIC):
+    case(TYPE_BOOLEAN):
+    default:
+    {
+      return "=";
+      break;
+    }
+  }
+}
+
+
 Glib::ustring Field::get_name() const
 {
   return m_field_info.get_name();
@@ -393,31 +552,97 @@ void Field::init_map()
     m_map_gda_type_to_glom_type[Gnome::Gda::VALUE_TYPE_TIME] = TYPE_TIME;
     m_map_gda_type_to_glom_type[Gnome::Gda::VALUE_TYPE_DATE] = TYPE_DATE;
     m_map_gda_type_to_glom_type[Gnome::Gda::VALUE_TYPE_BOOLEAN] = TYPE_BOOLEAN;
-    
+    m_map_gda_type_to_glom_type[Gnome::Gda::VALUE_TYPE_BINARY] = TYPE_IMAGE;
+        
     m_map_glom_type_to_gda_type[TYPE_NUMERIC] = Gnome::Gda::VALUE_TYPE_NUMERIC;
     m_map_glom_type_to_gda_type[TYPE_TEXT] = Gnome::Gda::VALUE_TYPE_STRING;
     m_map_glom_type_to_gda_type[TYPE_TIME] = Gnome::Gda::VALUE_TYPE_TIME;
     m_map_glom_type_to_gda_type[TYPE_DATE] = Gnome::Gda::VALUE_TYPE_DATE;
     m_map_glom_type_to_gda_type[TYPE_BOOLEAN] = Gnome::Gda::VALUE_TYPE_BOOLEAN;
-    
+    m_map_glom_type_to_gda_type[TYPE_IMAGE] = Gnome::Gda::VALUE_TYPE_BINARY;
+        
     m_map_type_names_ui[TYPE_INVALID] = _("Invalid");
     m_map_type_names_ui[TYPE_NUMERIC] = _("Number");
     m_map_type_names_ui[TYPE_TEXT] = _("Text");
     m_map_type_names_ui[TYPE_TIME] = _("Time");
     m_map_type_names_ui[TYPE_DATE] = _("Date");    
     m_map_type_names_ui[TYPE_BOOLEAN] = _("Boolean");
+    m_map_type_names_ui[TYPE_IMAGE] = _("Image");
 
+    //Non-translated names used for the document:
     m_map_type_names[TYPE_INVALID] = "Invalid";
     m_map_type_names[TYPE_NUMERIC] = "Number";
     m_map_type_names[TYPE_TEXT] = "Text";
     m_map_type_names[TYPE_TIME] = "Time";
     m_map_type_names[TYPE_DATE] = "Date";
     m_map_type_names[TYPE_BOOLEAN] = "Boolean";
+    m_map_type_names[TYPE_IMAGE] = "Image";
     
+    
+    //Conversions:
+    //These are the conversions know to be supported by the used version of postgres
+    m_map_conversions.clear();
+  
+    type_list_conversion_targets list_conversions;
+  
+    //Numeric:
+    list_conversions.clear();
+    list_conversions.push_back(Field::TYPE_BOOLEAN);
+    list_conversions.push_back(Field::TYPE_TEXT);
+    list_conversions.push_back(Field::TYPE_DATE);
+    list_conversions.push_back(Field::TYPE_TIME);
+    m_map_conversions[Field::TYPE_NUMERIC] = list_conversions;
+    
+    //Text:
+    list_conversions.clear();
+    list_conversions.push_back(Field::TYPE_BOOLEAN);
+    list_conversions.push_back(Field::TYPE_NUMERIC);
+    list_conversions.push_back(Field::TYPE_DATE);
+    list_conversions.push_back(Field::TYPE_TIME);
+    m_map_conversions[Field::TYPE_TEXT] = list_conversions;
+    
+    //Boolean:
+    list_conversions.clear();
+    list_conversions.push_back(Field::TYPE_TEXT);
+    list_conversions.push_back(Field::TYPE_NUMERIC);
+    list_conversions.push_back(Field::TYPE_DATE);
+    list_conversions.push_back(Field::TYPE_TIME);
+    m_map_conversions[Field::TYPE_BOOLEAN] = list_conversions;
+    
+    //Date:
+    list_conversions.clear();
+    list_conversions.push_back(Field::TYPE_TEXT);
+    list_conversions.push_back(Field::TYPE_NUMERIC);
+    list_conversions.push_back(Field::TYPE_BOOLEAN);
+    m_map_conversions[Field::TYPE_DATE] = list_conversions;
+    
+    //Time:
+    list_conversions.clear();
+    list_conversions.push_back(Field::TYPE_TEXT);
+    list_conversions.push_back(Field::TYPE_NUMERIC);
+    list_conversions.push_back(Field::TYPE_BOOLEAN);
+    m_map_conversions[Field::TYPE_TIME] = list_conversions;
+    
+  
     m_maps_inited = true;
   }
 }
 
+//static:
+bool Field::get_conversion_possible(glom_field_type field_type_src, glom_field_type field_type_dest)
+{
+  type_map_conversions::const_iterator iterFind = m_map_conversions.find(field_type_src);
+  if(iterFind != m_map_conversions.end())
+  {
+    const type_list_conversion_targets list_conversions = iterFind->second;
+    type_list_conversion_targets::const_iterator iterConversionFind = std::find(list_conversions.begin(), list_conversions.end(), field_type_dest);
+    if(iterConversionFind != list_conversions.end())
+      return true; //Success: conversion found.
+  }
+  
+  return false; //failed.
+}
+
 //static:
 Field::type_map_type_names Field::get_type_names_ui()
 {
@@ -486,6 +711,11 @@ void Field::set_calculation(const Glib::ustring& calculation)
   m_calculation = calculation;
 }
 
+bool Field::get_has_calculation() const
+{
+  return !m_calculation.empty();
+}
+
 Field::type_list_strings Field::get_calculation_fields() const
 {
   //TODO: Use regex, for instance with pcre here?
